/*
 *************************************************************************
 *
 * serdisp_control.h
 * routines for controlling serial lc-displays (eg: optrex323, nokia displays, ... )
 *
 *************************************************************************
 *
 * copyright (C) 2003-2006  wolfgang astleitner
 * email     mrwastl@users.sourceforge.net
 *
 *************************************************************************
 *
 * initially based on:
 *   http://www.thiemo.net/projects/orpheus/optrex/
 *
 *************************************************************************
 * This program is free software; you can redistribute it and/or modify   
 * it under the terms of the GNU General Public License as published by   
 * the Free Software Foundation; either version 2 of the License, or (at  
 * your option) any later version.                                        
 *                                                                        
 * This program is distributed in the hope that it will be useful, but    
 * WITHOUT ANY WARRANTY; without even the implied warranty of             
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU      
 * General Public License for more details.                               
 *                                                                        
 * You should have received a copy of the GNU General Public License      
 * along with this program; if not, write to the Free Software            
 * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA              
 * 02111-1307, USA.  Or, point your browser to                            
 * http://www.gnu.org/copyleft/gpl.html                                   
 *************************************************************************
 */

#ifndef SERDISP_CONTROL_H
#define SERDISP_CONTROL_H

/* version information */
#define SERDISP_VERSION_MAJOR 1
#define SERDISP_VERSION_MINOR 97
#define SERDISP_VERSION_EXTRA 7


#define SERDISP_VERSION(_major,_minor) (((_major) << 8) + (_minor))
#define SERDISP_VERSION_CODE (SERDISP_VERSION((SERDISP_VERSION_MAJOR),(SERDISP_VERSION_MINOR)))

#define SERDISP_VERSION_GET_MAJOR(_code)  ((int)( (_code) >> 8 ))                                                                         
#define SERDISP_VERSION_GET_MINOR(_code)  ((int)( (_code) & 0xFF ))                                                                       



#include "serdisp_connect.h"

/* define 'byte' if not available yet */
#ifndef byte
  #define byte unsigned char
#endif

#ifndef BOOL
  #define BOOL int
#endif


/* max. amount of contrast steps */
#define MAX_CONTRASTSTEP    10


#define SD_OPTION_NO        0
#define SD_OPTION_YES       1
#define SD_OPTION_TOGGLE    2

#define SD_OPTIONFLAG_RO    0x0
#define SD_OPTIONFLAG_RW    0x1
#define SD_OPTIONFLAG_STD   0x3

/* defines for GPIs/GPOs (general purpose inputs/outputs) */
#define SDGP_DIGITAL        0    /* digital: 0: off, <> 0: on */
#define SDGP_ANALOGUE       1    /* analogue: [min, max] == value */

#define SDGP_GLOBAL         0    /* global, display independent GPO/GPI (to be done) */
#define SDGP_LOCAL          1    /* local, display dependend GPO/GPI */


/*
 * features for serdisp_feature()
 * if a feature is not supported by a certain display:
 * it will be silently ignored
 */
/* note that serdisp_feature() and the following defines are deprecated. use serdisp_setoption() instead */
/* old (deprecated) */
#define FEATURE_NO          SD_OPTION_NO
#define FEATURE_YES         SD_OPTION_YES
#define FEATURE_TOGGLE      SD_OPTION_TOGGLE

#define FEATURE_CONTRAST    0x01
#define FEATURE_INVERT      0x02
#define FEATURE_BACKLIGHT   0x03
#define FEATURE_ROTATE      0x04
#define FEATURE_REVERSE     FEATURE_INVERT




typedef struct serdisp_options_s {
  char* name;               /* name of option */
  char* aliasnames;         /* alias names for this option (short name, ...) */
  long  minval, maxval;     /* mininum and maximum bandwidth for value */
  long  modulo;             /* option value MODULO modulo => 0 */
  int   flag;               /* standardoptions:  valid and r/w for all drivers (= SD_OPTIONFLAG_STD), 0 else
                               driver specific options: 
                                 option can be changed interactively (= SD_OPTIONFLAG_RW) or 
                                 only during setup (= SD_OPTIONFLAG_RO) */
  char* defines;            /* defines for option values (eg: 1=YES), separated by commas */
} serdisp_options_t;



#if 0
typedef struct serdisp_GPO_s {
  int   id;                 /* unique id */
  char* name;               /* name of option */
  char* aliasnames;         /* alias names for this option (short name, ...) */
  int   scope;              /* local (SDGP_LOCAL) or global (SDGP_GLOBAL) */
  int   type;               /* digital (SDGP_DIGITAL) or analogue (SDGP_ANALOGUE) */
  int   actlow;             /* only if digital: if GPO is active low: 1, 0 else */
  long  minval, maxval;     /* mininum and maximum bandwidth for value (digital: min=0, max=1) */
  long  mask;               /* bitmask (for cumulative setting of GPOs) */
  char* defines;            /* defines for option values (eg: 1=YES), separated by commas */
} serdisp_GPO_t;
#endif



/* struct containing all infos and other things concerning a display */
typedef struct serdisp_s {
  /* set on init. */
  char* dsp_name;           /* display name */
  char* dsp_optionstring;   /* option string */
  int   dsp_id;             /* display-ID to differ display models within a driver */

  int   width;              /* width of display in pixels */
  int   height;             /* height of display in pixels */
  int   depth;              /* colour depth (1: monochrome) */
  int   startxcol;          /* offset, if display-area is smaller than controller supported area */
  int   startycol;          /*   and display area doesn't start at (0,0) (eg: nokia 7110)   */

  int*  xreloctab;          /* relocation-tables containing logical x/y-pos to physical position    */
  int*  yreloctab;          /*   eg:  log. x-pos = 0 but x-pos needed to be passed to the controller = 8 => 
                                 xreloctab[0] = 8;    setpixel(x,y, ...) {  x_i = xreloctab[x]; ... } */
  int   xcolgaps;           /* if relocation-table(s) for accessing x/y pos needs more than width/height elements:  */
  int   ycolgaps;           /*   gap contains amount of these extra elements needed (default: 0) */ 
                            /*   these gaps are needed by optrex 323 display */

  long  dsparea_width;      /* display area: width and height of display area in micrometres (not millimetres to avoid float) */
  long  dsparea_height;     /*   optional. if dimensions are unknown values should be left 0 */

  BOOL  feature_contrast;   /* is contrast adjustable? */
  BOOL  feature_backlight;  /* backlight available? */
  BOOL  feature_invert;     /* is hardware inversion of display supported? */
  int   min_contrast;       /* if feature_contrast = true =>                                  */
  int   max_contrast;       /* min/max_contrast contain borders in which display is _usable_! */
                            /* nota bene: min_contrast may contain values > 0 !! */
                            /* eg: nokia pcd8544 displays aren't readable at all w/ values < 50 */

  long  delay;              /* delay after each write-operation (in nanoseconds).
                               ns = 0: ignore (no delay)
                               ns = 1: delay = one call of gettimeofday()
                               ns > 1: delay (at least) ns nanoseconds
                             */

  int   optalgo_maxdelta;   /* used for optimised transfer algorithm */

  void* specific_data;      /* place for a specific display-dependend struct (eg: i2c-ids, ...) */
  
  long* ctable;             /* colour table for indexed colour table */
  int*  greytable;          /* look up table for grey values:  grey value -> index */

  long  colour_spaces;      /* colour spaces supported by this display (bit field). 0 == auto-detect */
  
  long  default_bgcolour;   /* default background colour for displays with depth > 1 (will be used for ex. by serdisp_clear() */

  /* runtime settings */
  serdisp_CONN_t* sdcd;     /* device handle */
  int   connection_types;   /* connection types supported by this display (bit field) */

  int   curr_rotate;        /* display normal (0) or rotated 180 degrees (1) */
  int   curr_contrast;      /* current value for contrast */
  int   curr_backlight;     /* status of backlight */
  int   curr_invert;        /* display inverted (1) or not (0) */

  int   dbg_cnt;            /* counter. may be used for debugging issues */

  /* function pointers to display-specific functions
     (or: what to do if you think c++ is a horrible, unusable language but you need some features of OO-programming) */
  /* we're using 'struct serdisp_s*' here instead of 'serdisp_t*' because of forward definition */
  void  (*fp_init)          (struct serdisp_s* dd);
  void  (*fp_update)        (struct serdisp_s* dd);
  int   (*fp_setoption)     (struct serdisp_s* dd, const char* option, long value);
  void  (*fp_close)         (struct serdisp_s* dd);
  void  (*fp_setpixel)      (struct serdisp_s* dd, int x, int y, long colour);
  long  (*fp_getpixel)      (struct serdisp_s* dd, int x, int y);
  long  (*fp_transcolour)   (struct serdisp_s* dd, long colour);
  long  (*fp_transgrey)     (struct serdisp_s* dd, byte greyvalue);
  long  (*fp_lookupcolour)  (struct serdisp_s* dd, long colour);
  byte  (*fp_lookupgrey)    (struct serdisp_s* dd, long colour);
  /* get pointer to a value specific for a driver (stored in specific_data). *typesize then contains the size in byte */
  /* 0 if no such values */
  void* (*fp_getvalueptr)   (struct serdisp_s* dd, const char* option, int* typesize);

  /* internal display buffer */
  byte* scrbuf;
  /* change tags for internal display buffer */
  byte* scrbuf_chg;

  int   scrbuf_size;
  int   scrbuf_chg_size;

  /* wiring stuff */
  serdisp_wiresignal_t*     wiresignals;
  serdisp_wiredef_t*        wiredefs;
  int                       amountwiresignals;
  int                       amountwiredefs;
  /* options supported by driver */
  serdisp_options_t*        options;
  int                       amountoptions;
#if 0
  /* GPI/GPO stuff (general purpose inputs/outpus) */
  serdisp_GPO_t*            GOPs;
  int                       amountGPOs;    /* amount of gpos available */
  long                      curr_GPOmask;  /* current gpo bitmask */
#endif
} serdisp_t;



/* serdisp_display_s is a reduced version of serdisp_setup_s (defined in serdisp_control.c)
   needed for querying supported displays (from outside)
*/
typedef struct serdisp_display_s {
  char* dispname;           /* display name */
  char* aliasnames;         /* alias names */
  char* optionstring;       /* default options used for initalisation or 
                               if describing an already instanciated display: options currently used */
  char* description;        /* description text */
} serdisp_display_t;




    

#ifdef __cplusplus
extern "C" {
#endif

  serdisp_t*      serdisp_init                   (serdisp_CONN_t*, const char dispname[], const char optionstring[]);

  long            serdisp_getversioncode         (void);

  int             serdisp_getwidth               (serdisp_t* dd);
  int             serdisp_getheight              (serdisp_t* dd);
  int             serdisp_getcolours             (serdisp_t* dd);
  int             serdisp_getdepth               (serdisp_t* dd);
  
  serdisp_CONN_t* serdisp_getSDCONN              (serdisp_t* dd);
  

  int             serdisp_getpixelaspect         (serdisp_t* dd);

  void            serdisp_close                  (serdisp_t* dd);
  void            serdisp_quit                   (serdisp_t* dd);
  void            serdisp_clear                  (serdisp_t* dd);
  void            serdisp_rewrite                (serdisp_t* dd);
  void            serdisp_update                 (serdisp_t* dd);
  void            serdisp_blink                  (serdisp_t* dd, int what, int cnt, int delta);

  void            serdisp_clearbuffer            (serdisp_t* dd);

  int             serdisp_reset                  (serdisp_t* dd);
  serdisp_t*      serdisp_fullreset              (serdisp_t* dd);

  void            serdisp_setpixel               (serdisp_t* dd, int x, int y, long colour);

  long            serdisp_getpixel               (serdisp_t* dd, int x, int y);

  /* options */
  void            serdisp_setoption              (serdisp_t* dd, const char* optionname, long value);
  long            serdisp_getoption              (serdisp_t* dd, const char* optionname, int* typesize);
  int             serdisp_isoption               (serdisp_t* dd, const char* optionname);
  int             serdisp_getoptiondescription   (serdisp_t* dd, const char* optionname, serdisp_options_t* optiondesc);
  int             serdisp_nextoptiondescription  (serdisp_t* dd, serdisp_options_t* optiondesc);

  /* display info */
  const char*     serdisp_getdisplayname         (serdisp_t* dd);
  int             serdisp_getdisplaydescription  (const char* displayname, serdisp_display_t* displaydesc);
  int             serdisp_nextdisplaydescription (serdisp_display_t* displaydesc);
  void            serdisp_currdisplaydescription (serdisp_t* dd, serdisp_display_t* displaydesc);
  int             serdisp_nextwiringdescription  (const char* displayname, serdisp_wiredef_t* wiredesc);
  /* int             serdisp_isdisplay              (const char* displayname);*/

#if 0
  /* GPO */
  void            serdisp_setGPO                 (serdisp_t* dd, const char* name, long value);
  long            serdisp_getGPO                 (serdisp_t* dd, const char* name);
  void            serdisp_setGPObyID             (serdisp_t* dd, int id, long value);
  long            serdisp_getGPObyID             (serdisp_t* dd, int id);
  void            serdisp_setGPOcumulative       (serdisp_t* dd, long mask);
  long            serdisp_getGPOcumulative       (serdisp_t* dd);
  int             serdisp_isGPO                  (serdisp_t* dd, const char* name);
  int             serdisp_getGPOdescription      (serdisp_t* dd, const char* name, serdisp_options_t* optiondesc);
  int             serdisp_nextGPOdescription     (serdisp_t* dd, serdisp_GPO_t* GPOdesc);
#endif

  /* deprecated */
  void            serdisp_feature                (serdisp_t* dd, int feature, int value);
  void            serdisp_setpixels              (serdisp_t* dd, int x, int y, int w, int h, byte* data);

  /* internal use only */
  int             serdisp_setupoptions           (serdisp_t* dd, const char* dispname, const char* optionstring);

  int             serdisp_getdispindex           (const char* dispname);
  int             serdisp_comparedispnames       (const char* dispname1, const char* dispname2);
  int             serdisp_compareoptionnames     (serdisp_t* dd, const char* optionname1, const char* optionname2);
  

/* driver-specific setup functions */
  serdisp_t*      serdisp_i2c_setup              (const serdisp_CONN_t* sdcd, const char* dispname, const char* paramstring);
  serdisp_t*      serdisp_ks0108_setup           (const serdisp_CONN_t* sdcd, const char* dispname, const char* paramstring);
  serdisp_t*      serdisp_lh155_setup            (const serdisp_CONN_t* sdcd, const char* dispname, const char* paramstring);
  serdisp_t*      serdisp_nokcol_setup           (const serdisp_CONN_t* sdcd, const char* dispname, const char* paramstring);
  serdisp_t*      serdisp_pcd8544_setup          (const serdisp_CONN_t* sdcd, const char* dispname, const char* paramstring);
  serdisp_t*      serdisp_sed133x_setup          (const serdisp_CONN_t* sdcd, const char* dispname, const char* paramstring);
  serdisp_t*      serdisp_sed153x_setup          (const serdisp_CONN_t* sdcd, const char* dispname, const char* paramstring);
  serdisp_t*      serdisp_sed156x_setup          (const serdisp_CONN_t* sdcd, const char* dispname, const char* paramstring);
  serdisp_t*      serdisp_ssdoled_setup          (const serdisp_CONN_t* sdcd, const char* dispname, const char* paramstring);
  serdisp_t*      serdisp_t6963_setup            (const serdisp_CONN_t* sdcd, const char* dispname, const char* paramstring);
  serdisp_t*      serdisp_alphacool_setup        (const serdisp_CONN_t* sdcd, const char* dispname, const char* paramstring);
  serdisp_t*      serdisp_l4m_setup              (const serdisp_CONN_t* sdcd, const char* dispname, const char* paramstring);
#ifdef __cplusplus
    }
#endif


/* some macros */

#define serdisp_setupstructinfos(_dd,_signals,_defs,_options) \
            {(_dd)->wiresignals = (_signals); \
             (_dd)->amountwiresignals = (sizeof((_signals))/sizeof(serdisp_wiresignal_t));\
             (_dd)->wiredefs = (_defs); \
             (_dd)->amountwiredefs = (sizeof((_defs))/sizeof(serdisp_wiredef_t));\
             (_dd)->options = (_options); \
             (_dd)->amountoptions = (sizeof((_options))/sizeof(serdisp_options_t));\
            }


/* calculate distance between two pointers -> returns distance in bytes */
#define serdisp_ptrdistance(_endptr, _startptr)      ((int)((long)(_endptr) - (long)(_startptr)))

/* calculate length of a string starting at _startptr and ending at _endptr -> returns length in characters */
#define serdisp_ptrstrlen(_endptr, _startptr)      ( serdisp_ptrdistance( (_endptr), (_startptr) ) / sizeof(char))



#endif /* SERDISP_CONTROL_H */
